from __future__ import absolute_import, division, print_function

# Bindings for Falu
# API docs at https://falu.io/docs/api

api_key = None
app_information = None

# Set to either 'debug' or 'info',
log = None

# API services
from falu.services import *
